specifying_both_return_value_and_exception_is_not_allowed = "Specifying both return_value and exception is not allowed"
either_key_or_address_of_the_child_must_be_specified_and_not_both = "Either key or address of the child must be specified and not both"
